// $Id: MyDetectorConstruction.cc,v 1.1 2007/07/19 12:46:59 witoldp Exp $
// Flugg tag $Name: flugg_2007_2 $

#include "MyDetectorConstruction.hh"

#include "G4Material.hh"
#include "G4MaterialTable.hh"
#include "G4Element.hh"
#include "G4ElementTable.hh"
#include "G4Box.hh"
#include "G4Sphere.hh"
#include "G4LogicalVolume.hh"
#include "G4RotationMatrix.hh"
#include "G4ThreeVector.hh"
#include "G4Transform3D.hh"
#include "G4PVPlacement.hh"

MyDetectorConstruction::MyDetectorConstruction()
{
  expHall_x = 10.*cm;
  expHall_y = 10.*cm;
  expHall_z = 10.*cm;

  myBox_x = 10.*cm;
  myBox_y = 10.*cm;
  myBox_zA = 5.*cm;
  myBox_zB = 0.00841665*cm;
  myBox_zC = 0.0010873*cm;
  myBox_zD = 0.07640375*cm;
  myBox_zE = 4.9140923*cm;

}

MyDetectorConstruction::~MyDetectorConstruction()
{ }

G4VPhysicalVolume* MyDetectorConstruction::Construct()
{
  DefineMaterials();

//===================================volumes

  //------------------------------ experimental hall
  G4Box * experimentalHall_box
    = new G4Box("expHall_b",expHall_x,expHall_y,expHall_z);
  G4LogicalVolume * experimentalHall_log
    = new G4LogicalVolume(experimentalHall_box,defaultMaterial,"expHall_L",0,0,0);
  G4VPhysicalVolume * experimentalHall_phys
    = new G4PVPlacement(0,G4ThreeVector(),"expHall_P",
                        experimentalHall_log,0,false,0);

  //------------------------------ test box2
  G4Box * test_box2
    = new G4Box("myBox_b2",myBox_x,myBox_y,myBox_zA);
  G4LogicalVolume * test_log2
    = new G4LogicalVolume(test_box2,defaultMaterial,"myBox_L2",0,0,0);
  G4VPhysicalVolume * test_phys2
    = new G4PVPlacement(0,G4ThreeVector(0.,0.,-5.*cm),"myBox_P2",
                        test_log2,experimentalHall_phys,false,0);

  //------------------------------ test box3
  G4Box * test_box3
    = new G4Box("myBox_b3",myBox_x,myBox_y,myBox_zB);
  G4LogicalVolume * test_log3
    = new G4LogicalVolume(test_box3,FirstMaterial,"myBox_L3",0,0,0);
  G4VPhysicalVolume * test_phys3
    = new G4PVPlacement(0,G4ThreeVector(0.,0.,0.00841665*cm),"myBox_P3",
                        test_log3,experimentalHall_phys,false,0);

  //------------------------------ test box4
  G4Box * test_box4
    = new G4Box("myBox_b4",myBox_x,myBox_y,myBox_zC);
  G4LogicalVolume * test_log4
    = new G4LogicalVolume(test_box4,SecondMaterial,"myBox_L4",0,0,0);
  G4VPhysicalVolume * test_phys4
    = new G4PVPlacement(0,G4ThreeVector(0.,0.,0.0179206*cm),"myBox_P4",
                        test_log4,experimentalHall_phys,false,0);

  //------------------------------ test box5
  G4Box * test_box5
    = new G4Box("myBox_b5",myBox_x,myBox_y,myBox_zD);
  G4LogicalVolume * test_log5
    = new G4LogicalVolume(test_box5,ThirdMaterial,"myBox_L5",0,0,0);
  G4VPhysicalVolume * test_phys5
    = new G4PVPlacement(0,G4ThreeVector(0.,0.,0.09541165*cm),"myBox_P5",
                        test_log5,experimentalHall_phys,false,0);

  //------------------------------ test box6
  G4Box * test_box6
    = new G4Box("myBox_b6",myBox_x,myBox_y,myBox_zE);
  G4LogicalVolume * test_log6
    = new G4LogicalVolume(test_box6,defaultMaterial,"myBox_L6",0,0,0);
  G4VPhysicalVolume * test_phys6
    = new G4PVPlacement(0,G4ThreeVector(0.,0.,5.0859077*cm),"myBox_P6",
                        test_log6,experimentalHall_phys,false,0);

  //------------------------------------------------------------------

  return experimentalHall_phys;
}

void MyDetectorConstruction::DefineMaterials()
{ 
G4String name, symbol;             //a=mass of a mole;
G4double a, z, density;            //z=mean number of protons;  
G4double temperature, pressure;

density = 2.700*g/cm3;
a = 26.98*g/mole;
G4Material* Al = new G4Material(name="Aluminum", z=13., a, density);

density = 19.300*g/cm3;
a = 196.97*g/mole;
G4Material* Au = new G4Material(name="Gold", z=79., a, density);

//
// vacuum
//

density     = universe_mean_density;    //from PhysicalConstants.h
pressure    = 3.e-18*pascal;
temperature = 2.73*kelvin;
G4Material* Void =new G4Material(name="Galactic", z=1., a=1.01*g/mole, density,
                   kStateGas,temperature,pressure);

 //default materials of the calorimeter
  FirstMaterial = Al;
  SecondMaterial = Au;
  ThirdMaterial = Al;
  defaultMaterial = Void;
}
